@@ -10,7 +10,7 @@ from typing import Iterable, List, Tuple
 
 class Segment(NamedTuple):
     """A piece of text with associated style.
-    
+
     Args:
         text (str): A piece of text.
         style (:class:`~rich.style.Style`, optional): An optional style to apply to the text.
@@ -90,7 +90,7 @@ class Segment(NamedTuple):
 
         Returns:
             Iterable[Segment]: And iterable of Segment instances.
-        
+
         """
         if is_control:
             return filter(attrgetter("is_control"), segments)
@@ -183,7 +183,7 @@ class Segment(NamedTuple):
             segments (Iterable[Segment]): A list of segments in a single line.
             length (int): The desired width of the line.
             style (Style, optional): The style of padding if used (space on the end). Defaults to None.
-            pad (bool, optional): Pad lines with spaces if they are shorter than `length`. Defaults to True. 
+            pad (bool, optional): Pad lines with spaces if they are shorter than `length`. Defaults to True.
 
         Returns:
             List[Segment]: A line of segments with the desired length.
@@ -216,10 +216,10 @@ class Segment(NamedTuple):
 
     @classmethod
     def get_line_length(cls, line: List["Segment"]) -> int:
-        r"""Get the length of list of segments.
+        """Get the length of list of segments.
 
         Args:
-            line (List[Segment]): A line encoded as a list of Segments (assumes no '\n' characters),
+            line (List[Segment]): A line encoded as a list of Segments (assumes no '\\n' characters),
 
         Returns:
             int: The length of the line.
@@ -228,10 +228,10 @@ class Segment(NamedTuple):
 
     @classmethod
     def get_shape(cls, lines: List[List["Segment"]]) -> Tuple[int, int]:
-        r"""Get the shape (enclosing rectangle) of a list of lines.
+        """Get the shape (enclosing rectangle) of a list of lines.
 
         Args:
-            lines (List[List[Segment]]): A list of lines (no '\n' characters).
+            lines (List[List[Segment]]): A list of lines (no '\\n' characters).
 
         Returns:
             Tuple[int, int]: Width and height in characters.
