@@ -28,9 +28,6 @@ if TYPE_CHECKING:
 class Column:
     """Defines a column in a table."""
 
-    index: int
-    """Index of column."""
-
     header: "RenderableType" = ""
     """RenderableType: Renderable for the header (typically a string)"""
 
@@ -60,6 +57,9 @@ class Column:
     no_wrap: bool = False
     """bool: Prevent wrapping of text within the column. Defaults to ``False``."""
 
+    _index: int = 0
+    """Index of column."""
+
     _cells: List["RenderableType"] = field(default_factory=list)
 
     @property
@@ -84,7 +84,7 @@ class _Cell(NamedTuple):
 
 class Table(JupyterMixin):
     """A console renderable to draw a table.
-    
+
     Args:
         *headers (Union[Column, str]): Column headers, either as a string, or :class:`~rich.table.Column` instance.
         title (Union[str, Text], optional): The title of the table rendered at the top. Defaults to None.
@@ -108,6 +108,8 @@ class Table(JupyterMixin):
         border_style (Union[str, Style], optional): Style of the border. Defaults to None.
         title_style (Union[str, Style], optional): Style of the title. Defaults to None.
         caption_style (Union[str, Style], optional): Style of the caption. Defaults to None.
+        title_justify (str, optional): Justify method for title. Defaults to "center".
+        caption_justify (str, optional): Justify method for caption. Defaults to "center".
     """
 
     columns: List[Column]
@@ -136,12 +138,19 @@ class Table(JupyterMixin):
         border_style: StyleType = None,
         title_style: StyleType = None,
         caption_style: StyleType = None,
+        title_justify: "JustifyMethod" = "center",
+        caption_justify: "JustifyMethod" = "center",
     ) -> None:
 
-        self.columns = [
-            (Column(index, header) if isinstance(header, str) else header)
-            for index, header in enumerate(headers)
-        ]
+        self.columns: List[Column] = []
+        append_column = self.columns.append
+        for index, header in enumerate(headers):
+            if isinstance(header, str):
+                append_column(Column(_index=index, header=header))
+            else:
+                header._index = index
+                append_column(header)
+
         self.title = title
         self.caption = caption
         self.width = width
@@ -162,6 +171,8 @@ class Table(JupyterMixin):
         self.border_style = border_style
         self.title_style = title_style
         self.caption_style = title_style
+        self.title_justify = title_justify
+        self.caption_justify = caption_justify
         self._row_count = 0
         self.row_styles = list(row_styles or [])
 
@@ -180,7 +191,7 @@ class Table(JupyterMixin):
             collapse_padding (bool, optional): Enable collapsing of padding around cells. Defaults to True.
             pad_edge (bool, optional): Enable padding around edges of table. Defaults to False.
             expand (bool, optional): Expand the table to fit the available space if ``True``, otherwise the table width will be auto-calculated. Defaults to False.
-        
+
         Returns:
             Table: A table instance.
         """
@@ -224,22 +235,29 @@ class Table(JupyterMixin):
         """Get the current row style."""
         if self.row_styles:
             return self.row_styles[index % len(self.row_styles)]
-        return Style()
+        return Style.null()
 
     def __rich_measure__(self, console: "Console", max_width: int) -> Measurement:
         if self.width is not None:
-            max_width = min(self.width, max_width)
+            max_width = self.width
 
-        if self.box:
-            max_width -= len(self.columns) - 1
-            if self.show_edge:
-                max_width -= 2
+        # if self.box:
+        #     max_width -= len(self.columns) - 1
+        #     if self.show_edge:
+        #         max_width -= 2
+
+        if max_width < 0:
+            return Measurement(0, 0)
 
         extra_width = self._extra_width
+
+        max_width = sum(self._calculate_column_widths(console, max_width))
+
         _measure_column = self._measure_column
 
         measurements = [
-            _measure_column(console, column, max_width) for column in self.columns
+            _measure_column(console, column, max_width - extra_width)
+            for column in self.columns
         ]
         minimum_width = (
             sum(measurement.minimum for measurement in measurements) + extra_width
@@ -277,7 +295,7 @@ class Table(JupyterMixin):
         no_wrap: bool = False,
     ) -> None:
         """Add a column to the table.
-        
+
         Args:
             header (RenderableType, optional): Text or renderable for the header.
                 Defaults to "".
@@ -293,7 +311,7 @@ class Table(JupyterMixin):
         """
 
         column = Column(
-            index=len(self.columns),
+            _index=len(self.columns),
             header=header,
             footer=footer,
             header_style=Style.pick_first(
@@ -315,7 +333,7 @@ class Table(JupyterMixin):
         self, *renderables: Optional["RenderableType"], style: StyleType = None
     ) -> None:
         """Add a row of renderables.
-        
+
         Args:
             *renderables (None or renderable): Each cell in a row must be a renderable object (including str),
                 or ``None`` for a blank cell.
@@ -340,7 +358,7 @@ class Table(JupyterMixin):
             ]
         for index, renderable in enumerate(cell_renderables):
             if index == len(columns):
-                column = Column(index)
+                column = Column(_index=index)
                 for _ in range(self._row_count):
                     add_cell(column, Text(""))
                 self.columns.append(column)
@@ -362,7 +380,7 @@ class Table(JupyterMixin):
 
         max_width = options.max_width
         if self.width is not None:
-            max_width = min(self.width, max_width)
+            max_width = self.width
         if self.box:
             max_width -= len(self.columns) - 1
             if self.show_edge:
@@ -372,23 +390,28 @@ class Table(JupyterMixin):
 
         render_options = options.update(width=table_width)
 
-        def render_annotation(text: TextType, style: StyleType) -> "RenderResult":
+        def render_annotation(
+            text: TextType, style: StyleType, justify: "JustifyMethod" = "center"
+        ) -> "RenderResult":
             render_text = (
                 console.render_str(text, style=style) if isinstance(text, str) else text
             )
             return console.render(
-                render_text, options=render_options.update(justify="center")
+                render_text, options=render_options.update(justify=justify)
             )
 
         if self.title:
             yield from render_annotation(
-                self.title, style=Style.pick_first(self.title_style, "table.title")
+                self.title,
+                style=Style.pick_first(self.title_style, "table.title"),
+                justify=self.title_justify,
             )
         yield from self._render(console, render_options, widths)
         if self.caption:
             yield from render_annotation(
                 self.caption,
                 style=Style.pick_first(self.caption_style, "table.caption"),
+                justify=self.caption_justify,
             )
 
     def _calculate_column_widths(self, console: "Console", max_width: int) -> List[int]:
@@ -408,7 +431,7 @@ class Table(JupyterMixin):
                     for _range, column in zip(width_ranges, columns)
                 ]
                 flex_minimum = [
-                    (column.width or 1) + get_padding_width(column.index)
+                    (column.width or 1) + get_padding_width(column._index)
                     for column in columns
                     if column.flexible
                 ]
@@ -432,6 +455,12 @@ class Table(JupyterMixin):
                 widths = ratio_reduce(excess_width, [1] * len(widths), widths, widths)
                 table_width = sum(widths)
 
+            width_ranges = [
+                self._measure_column(console, column, width)
+                for width, column in zip(widths, columns)
+            ]
+            widths = [_range.maximum or 1 for _range in width_ranges]
+
         if table_width < max_width and self.expand:
             pad_widths = ratio_distribute(max_width - table_width, widths)
             widths = [_width + pad for _width, pad in zip(widths, pad_widths)]
@@ -540,7 +569,10 @@ class Table(JupyterMixin):
     ) -> Measurement:
         """Get the minimum and maximum width of the column."""
 
-        padding_width = self._get_padding_width(column.index)
+        if max_width < 1:
+            return Measurement(0, 0)
+
+        padding_width = self._get_padding_width(column._index)
 
         if column.width is not None:
             # Fixed width column
@@ -553,7 +585,7 @@ class Table(JupyterMixin):
         append_min = min_widths.append
         append_max = max_widths.append
         get_render_width = Measurement.get
-        for cell in self._get_cells(column.index, column):
+        for cell in self._get_cells(column._index, column):
             _min, _max = get_render_width(console, cell.renderable, max_width)
             append_min(_min)
             append_max(_max)
@@ -626,7 +658,7 @@ class Table(JupyterMixin):
             max_height = 1
             cells: List[List[List[Segment]]] = []
             if header_row or footer_row:
-                row_style = Style()
+                row_style = Style.null()
             else:
                 row_style = get_style(
                     get_row_style(index - 1 if show_header else index)
